#include <stdio.h>
#include "wrapper.h"
#include "ParCALFuncs.h"

NhlErrorTypes puv2vrf_W( void )
{
	/*
	* Input array variables
	*/
	void *u, *v;
	int ndims_u;
	ng_size_t dsizes_u[NCL_MAX_DIMENSIONS];
	int ndims_v;
	ng_size_t dsizes_v[NCL_MAX_DIMENSIONS];
	NclScalar missing_u, missing_v, missing_du, missing_dv;
	NclBasicDataTypes type_u, type_v;
	int has_missing_u, has_missing_v, found_missing_u, found_missing_v;
	/*
	* Output array variables
	*/
	void *vort_shnd;
	NclMultiDValData vort = NULL;
	int ndims_vort;
	ng_size_t *dsizes_vort;
	NclBasicDataTypes type_vort;
	NclScalar missing_vort, missing_dvort;
	int has_missing_vort;

	/*
	* Misc stuff
	*/
	int i, retval;

	/*
	* Retrieve parameters
	*
	* Note any of the pointer parameters can be set to NULL, which
	* implies you don't care about its value.
	*/
	u = (void*)NclGetArgValue(
		   0,
		   3,
		   &ndims_u, 
		   dsizes_u,
		   &missing_u,
		   &has_missing_u,
		   &type_u,
		   DONT_CARE);

	v = (void*)NclGetArgValue(
		   1,
		   3,
		   &ndims_v, 
		   dsizes_v,
		   &missing_v,
		   &has_missing_v,
		   &type_v,
		   DONT_CARE);

	/*
	* Get output array.
	*/
	vort = NclGetArgData(
			2,
			3,
			1);
	ndims_vort = vort->multidval.n_dims;
	dsizes_vort = vort->multidval.dim_sizes;
	missing_vort = vort->multidval.missing_value.value;
	has_missing_vort = vort->multidval.missing_value.has_missing;
	type_vort = vort->multidval.data_type;

	/*
	* The grids coming in must be at least 2-dimensional and have the same # of
	* dimensions.
	*/
	if( ndims_u != ndims_v || ndims_u < 2 ) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrf: The input arrays must be at least 2-dimensional and have the same number of dimensions");
		return(NhlFATAL);
	}
	for( i = 0; i < ndims_u; i++ ) {
		if( dsizes_u[i] != dsizes_v[i] ) {
		  NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrf: The input arrays must have the same dimension sizes");
		  return(NhlFATAL);
		}
	}
	/*
	* The output array must also be at least 2-dimensional.
	*/
	if( ndims_vort != ndims_u  ) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrf: The input/output arrays must have the same number of dimensions");
		return(NhlFATAL);
	}
	for( i = 0; i < ndims_u; i++ ) {
		if( dsizes_vort[i] != dsizes_u[i] ) {
		  NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrf: The input/output arrays must have the same dimension sizes");
		  return(NhlFATAL);
		}
	}
	if(type_vort != NCL_float && type_vort != NCL_double && type_vort != NCL_pdouble) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrf: The output array must be float or double");
		return(NhlFATAL);
	}

	/* FIXME: We should also handle non-distributed data */
	vort_shnd = ParCALUVToVort(u, v);
	vort->data.shnd = vort_shnd;
	vort->multidval.data_type = NCL_pdouble;

	retval = NclSetArgSMode(2, 3, NCLOBJDISTRIB);
	if(retval != NhlNOERROR){
		NhlPError(NhlFATAL, NhlEUNKNOWN, "uv2vrf: Unable to set the output array as a distributed array");
		return NhlFATAL;
	}

	return(NhlNOERROR);
}


NhlErrorTypes puv2vrF_W( void )
{
/*
 * Input array variables
 */
  void *u, *v;
  int ndims_u;
  ng_size_t dsizes_u[NCL_MAX_DIMENSIONS];
  int ndims_v;
  ng_size_t dsizes_v[NCL_MAX_DIMENSIONS];
  NclBasicDataTypes type_u, type_v;
  NclScalar missing_u, missing_v;
  int has_missing_u, has_missing_v, found_missing_u, found_missing_v;
/*
 * Output array variables
 */
  void *vort;
  NclBasicDataTypes type_vort;
  NclScalar missing_vort, missing_dvort;

  int i;

/*
 * Retrieve parameters
 *
 * Note any of the pointer parameters can be set to NULL, which
 * implies you don't care about its value.
 */
  u = (void*)NclGetArgValue(
           0,
           2,
           &ndims_u, 
           dsizes_u,
           &missing_u,
           &has_missing_u,
           &type_u,
           DONT_CARE);
  v = (void*)NclGetArgValue(
           1,
           2,
           &ndims_v, 
           dsizes_v,
           &missing_v,
           &has_missing_v,
           &type_v,
           DONT_CARE);

/*
 * The grids coming in must be at least 2-dimensional and have the same # of
 * dimensions.
 */
  if( ndims_u != ndims_v || ndims_u < 2 ) {
    NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrF: The input arrays must be at least 2-dimensional and have the same number of dimensions");
    return(NhlFATAL);
  }
  for( i = 0; i < ndims_u; i++ ) {
    if( dsizes_u[i] != dsizes_v[i] ) {
      NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrF: The input arrays must have the same dimension sizes");
      return(NhlFATAL);
    }
  }

  /* FIXME: Check the types of u and v */

  type_vort = NCL_pdouble;
  /* FIXME: Handle missing values */

  vort = ParCALUVToVort(u, v);

  return(NclReturnValue(vort,ndims_u,dsizes_u,NULL,type_vort,0));
}


NhlErrorTypes puv2vrg_W( void )
{
	/*
	* Input array variables
	*/
	void *u, *v;
	int ndims_u;
	ng_size_t dsizes_u[NCL_MAX_DIMENSIONS];
	int ndims_v;
	ng_size_t dsizes_v[NCL_MAX_DIMENSIONS];
	NclScalar missing_u, missing_v, missing_du, missing_dv;
	NclBasicDataTypes type_u, type_v;
	int has_missing_u, has_missing_v, found_missing_u, found_missing_v;
	/*
	* Output array variables
	*/
	void *vort_shnd;
	NclMultiDValData vort = NULL;
	int ndims_vort;
	ng_size_t *dsizes_vort;
	NclBasicDataTypes type_vort;
	NclScalar missing_vort, missing_dvort;
	int has_missing_vort;

	/*
	* Misc stuff
	*/
	int i, retval;

	/*
	* Retrieve parameters
	*
	* Note any of the pointer parameters can be set to NULL, which
	* implies you don't care about its value.
	*/
	u = (void*)NclGetArgValue(
		   0,
		   3,
		   &ndims_u, 
		   dsizes_u,
		   &missing_u,
		   &has_missing_u,
		   &type_u,
		   DONT_CARE);

	v = (void*)NclGetArgValue(
		   1,
		   3,
		   &ndims_v, 
		   dsizes_v,
		   &missing_v,
		   &has_missing_v,
		   &type_v,
		   DONT_CARE);

	/*
	* Get output array.
	*/
	vort = NclGetArgData(
			2,
			3,
			1);
	ndims_vort = vort->multidval.n_dims;
	dsizes_vort = vort->multidval.dim_sizes;
	missing_vort = vort->multidval.missing_value.value;
	has_missing_vort = vort->multidval.missing_value.has_missing;
	type_vort = vort->multidval.data_type;
	/*
	vort = NclGetArgValue(
		   2,
		   3,
		   &ndims_vort, 
		   dsizes_vort,
		   &missing_vort,
		   &has_missing_vort,
		   &type_vort,
		   1);
	*/
	/*
	* The grids coming in must be at least 2-dimensional and have the same # of
	* dimensions.
	*/
	if( ndims_u != ndims_v || ndims_u < 2 ) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrg: The input arrays must be at least 2-dimensional and have the same number of dimensions");
		return(NhlFATAL);
	}
	for( i = 0; i < ndims_u; i++ ) {
		if( dsizes_u[i] != dsizes_v[i] ) {
		  NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrg: The input arrays must have the same dimension sizes");
		  return(NhlFATAL);
		}
	}
	/*
	* The output array must also be at least 2-dimensional.
	*/
	if( ndims_vort != ndims_u  ) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrg: The input/output arrays must have the same number of dimensions");
		return(NhlFATAL);
	}
	for( i = 0; i < ndims_u; i++ ) {
		if( dsizes_vort[i] != dsizes_u[i] ) {
		  NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrg: The input/output arrays must have the same dimension sizes");
		  return(NhlFATAL);
		}
	}
	if(type_vort != NCL_float && type_vort != NCL_double && type_vort != NCL_pdouble) {
		NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrg: The output array must be float or double");
		return(NhlFATAL);
	}

	/* FIXME: We should also handle non-distributed data */
	vort_shnd = ParCALUVToVort(u, v);
	vort->data.shnd = vort_shnd;
	vort->multidval.data_type = NCL_pdouble;

	retval = NclSetArgSMode(2, 3, NCLOBJDISTRIB);
	if(retval != NhlNOERROR){
		NhlPError(NhlFATAL, NhlEUNKNOWN, "uv2vrg: Unable to set the output array as a distributed array");
		return NhlFATAL;
	}

	return(NhlNOERROR);
}


NhlErrorTypes puv2vrG_W( void )
{
/*
 * Input array variables
 */
  void *u, *v;
  int ndims_u;
  ng_size_t dsizes_u[NCL_MAX_DIMENSIONS];
  int ndims_v;
  ng_size_t dsizes_v[NCL_MAX_DIMENSIONS];
  NclBasicDataTypes type_u, type_v;
  NclScalar missing_u, missing_v;
  int has_missing_u, has_missing_v, found_missing_u, found_missing_v;
/*
 * Output array variables
 */
  void *vort;
  NclBasicDataTypes type_vort;
  NclScalar missing_vort, missing_dvort;

  int i;

/*
 * Retrieve parameters
 *
 * Note any of the pointer parameters can be set to NULL, which
 * implies you don't care about its value.
 */
  u = (void*)NclGetArgValue(
           0,
           2,
           &ndims_u, 
           dsizes_u,
           &missing_u,
           &has_missing_u,
           &type_u,
           DONT_CARE);
  v = (void*)NclGetArgValue(
           1,
           2,
           &ndims_v, 
           dsizes_v,
           &missing_v,
           &has_missing_v,
           &type_v,
           DONT_CARE);

/*
 * The grids coming in must be at least 2-dimensional and have the same # of
 * dimensions.
 */
  if( ndims_u != ndims_v || ndims_u < 2 ) {
    NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrG: The input arrays must be at least 2-dimensional and have the same number of dimensions");
    return(NhlFATAL);
  }
  for( i = 0; i < ndims_u; i++ ) {
    if( dsizes_u[i] != dsizes_v[i] ) {
      NhlPError(NhlFATAL,NhlEUNKNOWN,"uv2vrG: The input arrays must have the same dimension sizes");
      return(NhlFATAL);
    }
  }

  type_vort = NCL_pdouble;
  /* FIXME: Handle missing values */

  vort = ParCALUVToVort(u, v);

  return(NclReturnValue(vort,ndims_u,dsizes_u,NULL,type_vort,0));
}
