/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef MBMesh_hpp
#define MBMesh_hpp

#define MPICH_SKIP_MPICXX
#include "mpi.h"
#include "moab/Core.hpp"
#include "moab/Types.hpp"
#include "MeshFInfo.hpp"

namespace parncl{

//FIXME: Is there a way around this?
class ParCALMeshVal;

class MBMesh{
	public:
		~MBMesh();
		virtual bool ctest(const std::vector<size_t>& var_dims) = 0;
		virtual bool ctest(ParCALMeshVal &val, NclSelectionRecord *sel) = 0;

		/* Getters/Setters etc */
		void add_mesh_val(parncl::ParCALMeshVal *mesh_val);
		void del_mesh_val(parncl::ParCALMeshVal *mesh_val);
		MeshFInfo &get_mesh_finfo(void ) const;
		moab::Interface *const get_mb_core(void ) const;
		moab::EntityHandle get_mb_mset(void ) const;
		MPI_Comm get_mpi_comm(void ) const;
		int get_mpi_rank(void ) const;
		std::string get_part_method(void ) const;

		void inc_ref_cnt(void );
		void dec_ref_cnt(void );
		int get_ref_cnt(void ) const;
		void set_ref_cnt(int cnt);

	protected:
		// FIXME: Some of this info might be in parcal file info - so we might not
		// want to cache them
		MBMesh(moab::Interface *mb_gcore, MPI_Comm mb_comm, moab::ParallelComm *mb_pcomm, MeshFInfo *mesh_finfo, const std::string &part_method);

		MeshFInfo *mesh_finfo_;

		moab::Interface *const mb_gcore_;
		MPI_Comm mb_comm_;
		moab::ParallelComm *mb_pcomm_;

		int mb_rank_;
		std::string part_method_;

	private:
		MBMesh();

		/* Just an aggregation of mesh vals here */
		std::list<parncl::ParCALMeshVal *> mesh_vals_;

		int ref_cnt_;
};

} // namespace parncl

#endif // MBMesh_hpp
