/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#include "MeshTypes.hpp"
#include <iostream>

namespace parncl{

// CAM Eul consts
const std::string CAM_Eul_ID_GLOBAL_ATT_NAMES[] = {std::string("source")};
const std::string CAM_Eul_ID_GLOBAL_ATT_VALS[] = {std::string("CAM")};
const std::string CAM_Eul_ID_GLOBAL_DIM_NAMES[] = {std::string("lon"), std::string("lat")};

// CAM FVCD consts
const std::string CAM_FV_ID_GLOBAL_ATT_NAMES[] = {std::string("source")};
const std::string CAM_FV_ID_GLOBAL_ATT_VALS[] = {std::string("CAM")};
const std::string CAM_FV_ID_GLOBAL_DIM_NAMES[] = {std::string("lon"), std::string("lat"), std::string("slon"), std::string("slat")};

// CAM HOMME consts
const std::string CAM_HOMME_ID_GLOBAL_ATT_NAMES[] = {std::string("source"), std::string("np")};
const std::string CAM_HOMME_ID_GLOBAL_ATT_VALS[] = {std::string("CAM")};
const std::string CAM_HOMME_ID_GLOBAL_DIM_NAMES[] = {};

// Functions
static bool type_of(const pargal::pcatt &att, const std::set<std::string> &dim_names, const std::vector<std::string> &att_id_names, const std::vector<std::string> &att_id_vals, const std::vector<std::string> &dim_id_names)
{
	bool is_mesh_type = true;

	std::set<std::string> att_names = att.get_att_names();

	std::vector<std::string>::const_iterator att_id_names_iter = att_id_names.begin();
	std::vector<std::string>::const_iterator att_id_vals_iter = att_id_vals.begin();
	for(; (att_id_names_iter != att_id_names.end()) && (att_id_vals_iter != att_id_vals.end()); ++att_id_names_iter, ++att_id_vals_iter){
		std::set<std::string>::const_iterator iter = std::find(att_names.begin(), att_names.end(), *att_id_names_iter);
		if(iter != att_names.end()){
			// FIXME: We only handle matching strings right now
			std::string att_val;
			assert(att.get_data_type(*iter) == std::string("char"));
			att.get_vals(*iter, att_val);
			if(att_val != *att_id_vals_iter){
				is_mesh_type = false;
				break;
			}
		}
		else{
			is_mesh_type = false;
			break;
		}
	}
	// ID Names with no vals
	for(; (att_id_names_iter != att_id_names.end()); ++att_id_names_iter){
		std::set<std::string>::const_iterator iter = std::find(att_names.begin(), att_names.end(), *att_id_names_iter);
		if(iter == att_names.end()){
			is_mesh_type = false;
		}
	}

	if(is_mesh_type){
		for(std::vector<std::string>::const_iterator iter = dim_id_names.begin(); iter != dim_id_names.end(); ++iter){
			if(std::find(dim_names.begin(), dim_names.end(), *iter) == dim_names.end()){
				is_mesh_type = false;
				break;
			}
		}
	}

	return is_mesh_type;
}

// MeshType class member defns

CAM_Eul_Mesh::CAM_Eul_Mesh(moab::Interface *mb_gcore, MPI_Comm mb_comm, moab::ParallelComm *mb_pcomm, MeshFInfo *mesh_finfo, const std::string &part_method):ScdMBMesh::ScdMBMesh(mb_gcore, mb_comm, mb_pcomm, mesh_finfo, part_method)
{
// Do we need to do anything here ?	
}

CAM_Eul_Mesh::~CAM_Eul_Mesh()
{
// FIXME: Do we need to do anything here ?	
}

bool CAM_Eul_Mesh::ctest(ParCALMeshVal &old_val, NclSelectionRecord *sel)
{
	return false;
}

bool CAM_Eul_Mesh::type_of(const pargal::pcatt &att, const std::set<std::string> &dim_names)
{
	const std::vector<std::string> att_id_names(CAM_Eul_ID_GLOBAL_ATT_NAMES, CAM_Eul_ID_GLOBAL_ATT_NAMES + sizeof(CAM_Eul_ID_GLOBAL_ATT_NAMES)/sizeof(std::string));
	const std::vector<std::string> att_id_vals(CAM_Eul_ID_GLOBAL_ATT_VALS, CAM_Eul_ID_GLOBAL_ATT_VALS + sizeof(CAM_Eul_ID_GLOBAL_ATT_VALS)/sizeof(std::string));
	const std::vector<std::string> dim_id_names(CAM_Eul_ID_GLOBAL_DIM_NAMES, CAM_Eul_ID_GLOBAL_DIM_NAMES + sizeof(CAM_Eul_ID_GLOBAL_DIM_NAMES)/sizeof(std::string));

	return parncl::type_of(att, dim_names, att_id_names, att_id_vals, dim_id_names);
}

CAM_FV_Mesh::CAM_FV_Mesh(moab::Interface *mb_gcore, MPI_Comm mb_comm, moab::ParallelComm *mb_pcomm, MeshFInfo *mesh_finfo, const std::string &part_method):ScdMBMesh::ScdMBMesh(mb_gcore, mb_comm, mb_pcomm, mesh_finfo, part_method)
{
// Do we need to do anything here ?	
}

bool CAM_FV_Mesh::type_of(const pargal::pcatt &att, const std::set<std::string> &dim_names)
{
	const std::vector<std::string> att_id_names(CAM_FV_ID_GLOBAL_ATT_NAMES, CAM_FV_ID_GLOBAL_ATT_NAMES + sizeof(CAM_FV_ID_GLOBAL_ATT_NAMES)/sizeof(std::string));
	const std::vector<std::string> att_id_vals(CAM_FV_ID_GLOBAL_ATT_VALS, CAM_FV_ID_GLOBAL_ATT_VALS + sizeof(CAM_FV_ID_GLOBAL_ATT_VALS)/sizeof(std::string));
	const std::vector<std::string> dim_id_names(CAM_FV_ID_GLOBAL_DIM_NAMES, CAM_FV_ID_GLOBAL_DIM_NAMES + sizeof(CAM_FV_ID_GLOBAL_DIM_NAMES)/sizeof(std::string));

	return parncl::type_of(att, dim_names, att_id_names, att_id_vals, dim_id_names);
}

CAM_HOMME_Mesh::CAM_HOMME_Mesh(moab::Interface *mb_gcore, MPI_Comm mb_comm, moab::ParallelComm *mb_pcomm, MeshFInfo *mesh_finfo, const std::string &part_method):UScdMBMesh::UScdMBMesh(mb_gcore, mb_comm, mb_pcomm, mesh_finfo, part_method)
{
// Do we need to do anything here ?	
}

bool CAM_HOMME_Mesh::type_of(const pargal::pcatt &att, const std::set<std::string> &dim_names)
{
	const std::vector<std::string> att_id_names(CAM_HOMME_ID_GLOBAL_ATT_NAMES, CAM_HOMME_ID_GLOBAL_ATT_NAMES + sizeof(CAM_HOMME_ID_GLOBAL_ATT_NAMES)/sizeof(std::string));
	const std::vector<std::string> att_id_vals(CAM_HOMME_ID_GLOBAL_ATT_VALS, CAM_HOMME_ID_GLOBAL_ATT_VALS + sizeof(CAM_HOMME_ID_GLOBAL_ATT_VALS)/sizeof(std::string));
	const std::vector<std::string> dim_id_names(CAM_HOMME_ID_GLOBAL_DIM_NAMES, CAM_HOMME_ID_GLOBAL_DIM_NAMES + sizeof(CAM_HOMME_ID_GLOBAL_DIM_NAMES)/sizeof(std::string));

	return parncl::type_of(att, dim_names, att_id_names, att_id_vals, dim_id_names);
}
} //namespace parncl
