/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef MeshTypes_hpp
#define MeshTypes_hpp

#include <vector>
#include <set>
#include <string>
#include "moab/Core.hpp"
#include "moab/ScdInterface.hpp"
#include "moab/HomXform.hpp"
#include "moab/EntityType.hpp"
#include "ScdMBMesh.hpp"
#include "UScdMBMesh.hpp"
#include "ParCALMeshVal.hpp"
#define MPICH_SKIP_MPICXX
#include "mpi.h"
extern "C"{
	#include "ParCALFuncs.h"
}

namespace parncl{

class CAM_Eul_Mesh : public ScdMBMesh{
	public:
		CAM_Eul_Mesh(moab::Interface *mb_gcore, MPI_Comm mb_comm, moab::ParallelComm *mb_pcomm, MeshFInfo *mesh_finfo, const std::string &part_method);
		~CAM_Eul_Mesh();

		virtual bool ctest(ParCALMeshVal &val, NclSelectionRecord *sel);
		static bool type_of(const pargal::pcatt &att, const std::set<std::string> &dim_names);

	private:
		CAM_Eul_Mesh();
};

class CAM_FV_Mesh : public ScdMBMesh{
	public:
		CAM_FV_Mesh(moab::Interface *mb_gcore, MPI_Comm mb_comm, moab::ParallelComm *mb_pcomm, MeshFInfo *mesh_finfo, const std::string &part_method);

		static bool type_of(const pargal::pcatt &att, const std::set<std::string> &dim_names);

	private:
		CAM_FV_Mesh();
};

class CAM_HOMME_Mesh : public UScdMBMesh{
	public:
		CAM_HOMME_Mesh(moab::Interface *mb_gcore, MPI_Comm mb_comm, moab::ParallelComm *mb_pcomm, MeshFInfo *mesh_finfo, const std::string &part_method);

		static bool type_of(const pargal::pcatt &att, const std::set<std::string> &dim_names);

	private:
		CAM_HOMME_Mesh();
};
} // namespace parncl

#endif // MeshTypes_hpp
