/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * 
 * Development of this software was sponsored by the Office of Biological  *
 * and Environmental Research of the U.S. Department of Energy's Office    *
 * of Science.                                                             *
 *                                                                         *
 * Copyright 2011 UChicago Argonne, LLC.                                   *
 *                                                                         *
 * Licensed under the Apache License, Version 2.0 (the "License"); you     *
 * may not use this file except in compliance with the License.  You may   *
 * obtain a copy of the License at                                         * 
 *                                                                         *
 *   http://www.apache.org/licenses/LICENSE-2.0                            *
 *                                                                         *
 * Unless required by applicable law or agreed to in writing, software     *
 * distributed under the License is distributed on an "AS IS" BASIS,       *
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or         *
 * implied.  See the License for the specific language governing           *
 * permissions and limitations under the License.                          *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */

#ifndef ParCALFuncs_h
#define ParCALFuncs_h

#ifdef NIO_LIB_ONLY
#include "niohlu.h"
#include "nioNresDB.h"
#else
#include <ncarg/hlu/hlu.h>
#include <ncarg/hlu/NresDB.h>
#endif
#include "defs.h"
#include "NclDataDefs.h"
#include "NclFileInterfaces.h"
#include "NclVar.h"
#include "VarSupport.h"
#include "NclData.h" 
#include "NclList.h"
/* #include "NclMultiDValData.h" */
#include "NclMdInc.h"
#include "ParNCLNetCdf.h"
#include "PNclUtils.h"

typedef enum{
	PARCAL_RD = 0x1,
	PARCAL_WR = 0x10
} ParCALAccessMode;

typedef enum{
	PARCAL_VAL_LOCAL = 1,
	PARCAL_VAL_GLOBAL
} ParCALValType;

/*
 * Initialize ParCAL module
 */
NhlErrorTypes ParCALInit(void );
/*
 * Finalize ParCAL module
 */
void ParCALFinalize(void );

/************** PARCAL File ops **************/
/*
 * Open a single file and get a ParCAL handle to it
 */
void *ParCALOpen(NclQuark file, int wr_status);
/*
 * Open multiple files and get a ParCAL handle to it
 */
NhlErrorTypes ParCALMOpen(void **pcal_hnds, int npcal_hnds, NclMultiDValData files, int wr_status);
/*
 * Close a ParCAL handle
 * The handle is closed when all files that are associated
 * with the handle issue a close.
 * Returns: 0 if all instances are closed, 1 otherwise
 */
int ParCALClose(void *pcal_hnd, NclQuark file);

/************** PARCAL Object funcs **************/

/* ParCAL funcs that allow creating new parcal objs etc */

/************** PARCAL ops **************/
/* Get meta-data from parcal */
int ParCALGetVarInfo(void *pcal_hnd, PNNCVarInqRecList **prec_list);
int ParCALGetDimInfo(void *pcal_hnd, PNNCDimInqRecList **prec_list);
int ParCALGetFileAttInfo(void *pcal_hnd, PNNCAttInqRecList **prec_list);

void *ParCALDupVar(void *p_var_shnd);
/* FIXME: While creating a variable, providing storage handle to another is ugly 
 * The interface should accept sz, dims, pcal_core_hnd etc 
*/
void *ParCALCreateVarFromVar(void *p_var_shnd, NclSelectionRecord *sel);
void *ParCALCacheVar(void **pcal_hnds, int npcal_hnds, NclQuark var_nameq, NclFVarSelRec *sel, int nsel);
int ParCALGetNumChunks(void *p_var_shnd);
/* FIXME: GetVal() should replace GetVar() */
void *ParCALGetVal(void *p_var_shnd, NclSelectionRecord *sel, ng_size_t req_chunk, ParCALValType val_type, NclTypeClass *out_val_type, ParCALAccessMode mode, ng_size_t *sz);
NhlErrorTypes ParCALFreeVar(void *p_var_shnd);
void ParCALClearValCache(void *p_var_shnd, ng_size_t req_chunk, ParCALValType val_type);

/* Calculate dim_avg_n() - Returns ptr to storage handle of the result */
void *ParCALDimAvg(NclMultiDValData md_val, int *dims, ng_size_t n_dims, NclSelectionRecord *sel);

void *ParCALUVToVort(void *u_pvar_shnd, void *v_pvar_shnd);

void *ParCALMin(NclMultiDValData md_val, int *dims, ng_size_t n_dims, NclSelectionRecord *sel, void *min_vals, ng_size_t n_min_vals, NclBasicDataTypes *otype);

void *ParCALMax(NclMultiDValData md_val, int *dims, ng_size_t n_dims, NclSelectionRecord *sel, void *max_vals, ng_size_t n_max_vals, NclBasicDataTypes *otype);

#endif
